#include <stm8s.h>
#include <stm8s_spi.h>
#include <stm8_uart.h>
#include <conversion.h>
#include <delay.h>

#define MISO 7 //SPI_MISO PC7
#define MOSI 6 //SPI_MOSI PC6
#define SCK 5 //SPI_SCK PC5
#define CS 3 //SPI_NSS PA3

void initSPI(void);
void SPITransmit(uint8_t data);
uint8_t SPIReceive(void);
void initADXL345(void);
void getValueFromRegister(uint8_t reg, uint8_t *val);
void getXYZValues(void);

int16_t x,y,z;

void initSPI(){
	GPIOC->DDR = (1<<MOSI) | (1<<SCK) | (0<<MISO); //MOSI & SCK output, MISO input
	GPIOC->CR1 = (1<<MOSI) | (1<<SCK); //MOSI& SCK push-pull
	
	GPIOA->DDR |= (1<<CS); //set CS as output
  GPIOA->CR1 |= (1<<CS); //set CS as push-pull
	GPIOA->ODR |= (1<<CS); //turn CS high
	
	SPI_DeInit();
	SPI_Init(
		SPI_FIRSTBIT_MSB,
		SPI_BAUDRATEPRESCALER_256,
		SPI_MODE_MASTER,
		SPI_CLOCKPOLARITY_HIGH,
		SPI_CLOCKPHASE_2EDGE,
		SPI_DATADIRECTION_2LINES_FULLDUPLEX,
		SPI_NSS_HARD,
		0x07);
	SPI_Cmd(ENABLE);
	
	//SPI->CR2 = 0b00000011; //2-line unidirectional, Hardware-Slave  management
	//SPI->CR1 = 0b01111111; //Enable CPHA, CPOL, MASTER; Baudrate: f/256, SPI Enable
}

void SPITransmit(uint8_t data){
	while(SPI_GetFlagStatus(SPI_FLAG_TXE) == RESET){
		nop(); //wait until TX-Buffer is free
	}
	SPI_SendData(data);
	while(SPI_GetFlagStatus(SPI_FLAG_BSY) == SET){
		nop(); //wait until not busy
	}
}
uint8_t SPIReceive(){
	uint8_t registerValue;
	while(SPI_GetFlagStatus(SPI_FLAG_BSY) == SET){
		nop(); //wait until not busy
	}
	while(SPI_GetFlagStatus(SPI_FLAG_RXNE) == SET){
		nop(); //wait until receive buffer is not empty
	}
	registerValue = SPI_ReceiveData();
	return registerValue;
}

void initADXL345(){
	//set FULL_RES & 16g-Range	
	GPIOA->ODR &= ~(1<<CS); //turn CS low
	SPITransmit(0x31);
	SPITransmit(0b00001011);
	GPIOA->ODR |= (1<<CS); //turn CS high
	
	delayms(15);
	
	//enable measurements
	GPIOA->ODR &= ~(1<<CS); //turn CS low
	SPITransmit(0x2D);
	SPITransmit(0x08);
	GPIOA->ODR |= (1<<CS); //turn CS high
}

void getValueFromRegister(uint8_t reg, uint8_t *val){
	uint8_t transmitValue = (0x80 | reg);
	uint8_t receivedValue = 0;
	
	GPIOA->ODR &= ~(1<<CS); //CS low
	SPITransmit(transmitValue); //write 'read' command
	receivedValue = SPIReceive();
	GPIOA->ODR |= (1<<CS); //CS high
	
	*val = receivedValue;
}

void getXYZValues(){
	uint8_t x0,x1,y0,y1,z0,z1;
	getValueFromRegister(0x32,&x0);
	getValueFromRegister(0x33,&x1);
	getValueFromRegister(0x34,&y0);
	getValueFromRegister(0x35,&y1);
	getValueFromRegister(0x36,&z0);
	getValueFromRegister(0x37,&z1);
	
	x = (x1<<8)|x0;
	y = (y1<<8)|y0;
	z = (z1<<8)|z0;
}

int main(){
	char c[255];
	uint8_t regVal;
	initUART();
	initSPI();
	initADXL345();
	while(1){
		initADXL345();
		delayms(125);
	}
} 