/* MAIN.C file*/

#include <stm8s.h>

#define CS 7 //PC7
#define SDIO 6 //PC6
#define SCLK 5 //PC5

void delay_ms(uint16_t ms);
char * ltoa(char * buf, long val, char pad, char padchar);
void initUART(void);

void UARTSend(char *message);
void UARTSendInt(uint8_t number);

void initPins(void);
void writeCommand(uint16_t cmd);
uint8_t readRegister(uint8_t reg);
void enableADXL345(void);
void getDEVID(void);
void selfTest(void);
void readXYZ(void);

int16_t x_val,y_val,z_val;
void delay_ms(uint16_t ms){
	while(ms--){
		uint16_t i;
		for(i = 0; i < ms; i++){
			nop();
		}
	}
}

char * ltoa(char * buf, long val, char pad, char padchar){
	char i;
	char neg = 0;
	char tmp_buf[21];

	if(val<0) {
		val = -val;
		neg = 1;
	}

	for(i=0; val>0; val/=10)
		tmp_buf[i++] = (val % 10) + '0';

	if(i==0)
		tmp_buf[i++] = '0';

	if(neg) {
		if(padchar) {
			tmp_buf[i++] = '-';
			while(i<pad)
				tmp_buf[i++] = ' ';
		}
		else {
			while(i<pad-1)
				tmp_buf[i++] = '0';
			tmp_buf[i++] = '-';
		}
	}
	else
		while(i<pad)
			tmp_buf[i++] = padchar ? ' ' : '0';

	while(i>0)
		*buf++ = tmp_buf[--i];

	*buf = 0;

	return buf;
}

void initUART(){
	uint16_t clockBaud = 0;
	uint8_t BRR2 = 0;
	uint8_t BRR1 = 0;
	
	CLK->CKDIVR = 0x00; //16MHz clock
	
	UART1->CR1 = (0<<2) | (0<<4); //Parity control disabled, 8 Data bits
	UART1->CR2 = (1<<2) | (1<<3); //enable Receive and Transmit
	UART1->CR3 = 0x00; //disable everything
	
	clockBaud = 16000000 / 19200; //16MHz, 19200Baud
	
	//mask appropriate bits
	BRR1 = (clockBaud >> 4) & (0x00FF);
	BRR2 = (clockBaud >> 8) & (0x00F0) | (clockBaud & (0x000F));
	
	UART1->BRR2 = BRR2;
	UART1->BRR1 = BRR1;
}

void UARTSend(char *message){
	char *ch = message;
	while(*ch){
		while(!(UART1->SR & (1<<6))); //Wait until transmission is complete
		UART1->DR = *ch;
		ch++;
	}
}

void UARTSendInt(uint8_t number){
	while(!(UART1->SR & (1<<6)));
	UART1->DR = number;
}

void initPins(){
	GPIOC->DDR = (1<<CS)| (1<<SDIO) | (1<<SCLK); //CS, SDIO, SCLK as output
	GPIOC->CR1 = (1<<CS) | (1<<SDIO) | (1<<SCLK); //CS, SDIO, SCLK as Push-pull
	GPIOC->ODR = (1<<CS) | (1<<SDIO) | (1<<SCLK); //turn CS, SDIO, SCLK on
}
void writeCommand(uint16_t cmd){
  uint8_t i;
	GPIOC->ODR &= ~(1<<CS); //turn CS low to start transmit
	delay_ms(2);
	GPIOC->DDR |= (1<<SDIO); //set SDIO as Output
	
	GPIOC->ODR |= (1<<SDIO); //turn SDIO high !ATTENTION!
	GPIOC->ODR |= (1<<SCLK); //turn SCLK high !ATTENTION!
	
  for(i = 0; i < 16; i++){ //continue writing
    GPIOC->ODR &= ~(1<<SCLK); 
    if(cmd & (1<<15)){ 
      GPIOC->ODR |= (1<<SDIO); 
    } else { 
      GPIOC->ODR &= ~(1<<SDIO); 
    }
    delay_ms(1);
    GPIOC->ODR |= (1<<SCLK); 
    delay_ms(1); 
    cmd <<= 1; 
  }
	
	GPIOC->ODR |= (1<<SDIO);

  GPIOC->ODR |= (1<<CS); //turn CS high to end transmit
}

uint8_t readRegister(uint8_t reg) {
	uint8_t i, k;
	uint8_t res = 0;
	GPIOC->ODR &= ~(1<<CS); //turn CS low to start transmit
	delay_ms(2);
	GPIOC->DDR |= (1<<SDIO); //set SDIO as Output
	GPIOC->ODR &= ~(1<<SDIO); //turn SDIO low !ATTENTION!
	delay_ms(1);
	for(i=0; i<8; i++) { //transmit 8 bits
			GPIOC->ODR &= ~(1<<SCLK);
			if(reg & (1<<7)) {            
					GPIOC->ODR |= (1<<SDIO);    
			} else {                        
					GPIOC->ODR &= ~(1<<SDIO); 
			}
			delay_ms(2);                   
			GPIOC->ODR |= (1<<SCLK);       
			delay_ms(2);                   
			reg <<= 1;                      
	}

	// read result
	GPIOC->DDR &= ~(1<<SDIO); // set SDIO as input
	for(k=0; k<8; k++) { //read 8 data bits
			res <<= 1;
			GPIOC->ODR &= ~(1<<SCLK);
			delay_ms(4);
			GPIOC->ODR |= (1<<SCLK);
			delay_ms(4);
			if(GPIOC->IDR & (1<<SDIO)) {  
					res |= 1;
			}
			delay_ms(4);
	}

	GPIOC->ODR |= (1<<CS); //turn CS high to end transmit
	return res;
}

void enableADXL345(){
	uint16_t write_command = 0;
	
	nop();
	write_command = ((0x00 | 0x31) << 8) | 0b00001011;//set SPI 4wire & 16g range & full-res
	writeCommand(write_command);
	delay_ms(10);
	write_command = ((0x00 | 0x2D) << 8) | 0x08; //Enable measurements
	writeCommand(write_command);
	//write_command = ((0x00 | 0x2C) << 8) | 0x0A; //100Hz data rate
	//writeCommand(write_command);
}

void selfTest(){
	uint8_t bValue = 0;
	uint8_t eValue = 0;
	uint16_t write_command = 0;
	
	bValue	= readRegister(0x80|0x2C);
	eValue = bValue | 0x80;
	write_command = ((0x00|0x2C)<<8) | eValue;
	
	writeCommand(write_command);
}

void getDEVID(){
	uint8_t devid = readRegister(0x80|0x00);
}

void readXYZ(){
	uint8_t x0 = 0;
	uint8_t x1 = 0;
	uint8_t y0 = 0;
	uint8_t y1 = 0;
	uint8_t z0 = 0;
	uint8_t z1 = 0;
	
	int16_t x = 0;
	int16_t y = 0;
	int16_t z = 0;
	
	//read X-Axis
	x0 = readRegister(0x80 | 0x32);
	nop();
	x1 = readRegister(0x80 | 0x33);
	nop();
	//read Y-Axis
	y0 = readRegister(0x80 | 0x34);
	nop();
	y1 = readRegister(0x80 | 0x35);
	nop();
	//read Z-Axis
	z0 = readRegister(0x80 | 0x36);
	nop();
	z1 = readRegister(0x80 | 0x37);
	nop();
	
	x = (int16_t)((x1 << 8) | x0);
	y = (int16_t)((y1 << 8) | y0);
	z = (int16_t)((z1 << 8) | z0);
	
	//+/- 16g mode & full-res: 4mg/LSB
	x_val = (x*4);
	y_val = (y*4);
	z_val = (z*4) - (1444+4096+2048+1024);
}

void SendXYZ(){
	char buff[128];
	readXYZ();
	ltoa(buff,x_val,1,'0');
	UARTSend("X: ");
	UARTSend(buff);
	
	ltoa(buff,y_val,1,'0');
	UARTSend(" Y: ");
	UARTSend(buff);
	
	ltoa(buff,z_val,1,'0');
	UARTSend(" Z: ");
	UARTSend(buff);
}

main()
{
	initPins();
	initUART();
	enableADXL345();
	do{
		enableADXL345();
		delay_ms(125);
	}while(1);
}